using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using Avalonia.Controls;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Logging;
using MPrakt.ViewModels;
using ReactiveUI;
using TourAgent.Models;
using TourAgent.Views;

namespace TourAgent.ViewModels
{
    public class HWindowViewModel : ViewModelBase
    {
        public static UP_SmirnovContext DB = new UP_SmirnovContext();

        UserControl us = new Views.Menu();

        public UserControl US
        {
            get => us;
            set => this.RaiseAndSetIfChanged(ref us, value);
        }

        MenuViewModel menuVM = new MenuViewModel();

        public MenuViewModel MenuVM
        {
            get => menuVM;
            set => menuVM = value;
        }

        TourViewModel _toursVM = new TourViewModel();

        public TourViewModel ToursVM
        {
            get => _toursVM;
            set => _toursVM = value;
        }

        OtelsViewModel _hotelsVM = new OtelsViewModel();

        public OtelsViewModel HotelsVM
        {
            get => _hotelsVM;
            set => _hotelsVM = value;
        }

        public void ChangePageToHotels()
        {
            US = new Hotels();
        }
        public void ChangePageToTours()
        {
            US = new Views.Tours();
        }

        public void ToMenu()
        {
            US = new Views.Menu();
        }
    }
    public class TourViewModel : ReactiveObject
	{

        UP_SmirnovContext DB = new UP_SmirnovContext();

		private List<Models.Tourss>? _tours;
        public List<Models.Tourss>? Tours { get => _tours; set => this.RaiseAndSetIfChanged(ref _tours, value); }

        private static List<string>? _types;
        public static List<string>? Types { get => _types; set => _types = value; }

        private static List<string>? _sort;
        public static List<string>? Sort { get => _sort; set => _sort = value; }

        private decimal _priceOfAll;
        public decimal PriceOfAll
        {
            get => _priceOfAll; 
            set => this.RaiseAndSetIfChanged(ref _priceOfAll, value);
        }

        private bool _checkOrNot = false;
        public bool CheckOrNot
        {
            get => _checkOrNot;
            set
            {
                _checkOrNot = value;
                Filter();
            }
        }

        private string? _findTour;
        public string? FindTour
        {
            get => _findTour;
            set
            {
                _findTour = value;
                Filter();
            }
        }

        private string? _selectedItem;
        public string? SelectedItem
        {
            get
            {
                return _selectedItem;
            }
            set
            {
                _selectedItem = value;
                Filter();
            }
        }

        private string? _selectedSort;
        public string? SelectedSort
        {
            get
            {
                return _selectedSort;
            }
            set
            {
                _selectedSort = value;
                Filter();
            }
        }

        public TourViewModel()
        {
            Tours = DB.Tours.ToList();
            Types = DB.Types.Select(x=>x.Name).ToList();
            Types.Insert(0, "");
            SelectedItem = Types[0];
            Sort = new List<string> { " ", "  ", "  " };
            SelectedSort = Sort[0];
        }

        public void Filter()
        {
            Tours = DB.Tours.ToList();
            PriceOfAll = 0;

            if (SelectedItem != "")
            {
                List <TypeOfTour> TypesForFilter = DB.TypeOfTours.ToList();
                List <int> IdForFilter = new List<int>();
                switch (SelectedItem)
                {
                    case " ":
                        TypesForFilter = TypesForFilter.Where(x=>x.TypeId==1).ToList();
                        IdForFilter = TypesForFilter.Select(x=>x.TourId).ToList();
                        Tours = Tours.Where(x=> IdForFilter.Contains(x.Id)).ToList();
                        break;
                    case " ":
                        TypesForFilter = TypesForFilter.Where(x => x.TypeId == 2).ToList();
                        IdForFilter = TypesForFilter.Select(x => x.TourId).ToList();
                        Tours = Tours.Where(x => IdForFilter.Contains(x.Id)).ToList();
                        break;
                    case "- ":
                        TypesForFilter = TypesForFilter.Where(x => x.TypeId == 3).ToList();
                        IdForFilter = TypesForFilter.Select(x => x.TourId).ToList();
                        Tours = Tours.Where(x => IdForFilter.Contains(x.Id)).ToList();
                        break;
                    case " ":
                        TypesForFilter = TypesForFilter.Where(x => x.TypeId == 4).ToList();
                        IdForFilter = TypesForFilter.Select(x => x.TourId).ToList();
                        Tours = Tours.Where(x => IdForFilter.Contains(x.Id)).ToList();
                        break;
                    case " ":
                        TypesForFilter = TypesForFilter.Where(x => x.TypeId == 5).ToList();
                        IdForFilter = TypesForFilter.Select(x => x.TourId).ToList();
                        Tours = Tours.Where(x => IdForFilter.Contains(x.Id)).ToList();
                        break;
                    case "  ":
                        TypesForFilter = TypesForFilter.Where(x => x.TypeId == 6).ToList();
                        IdForFilter = TypesForFilter.Select(x => x.TourId).ToList();
                        Tours = Tours.Where(x => IdForFilter.Contains(x.Id)).ToList();
                        break;
                    case "    ":
                        TypesForFilter = TypesForFilter.Where(x => x.TypeId == 7).ToList();
                        IdForFilter = TypesForFilter.Select(x => x.TourId).ToList();
                        Tours = Tours.Where(x => IdForFilter.Contains(x.Id)).ToList();
                        break;
                    case " ":
                        TypesForFilter = TypesForFilter.Where(x => x.TypeId == 8).ToList();
                        IdForFilter = TypesForFilter.Select(x => x.TourId).ToList();
                        Tours = Tours.Where(x => IdForFilter.Contains(x.Id)).ToList();
                        break;
                    case "- ":
                        TypesForFilter = TypesForFilter.Where(x => x.TypeId == 9).ToList();
                        IdForFilter = TypesForFilter.Select(x => x.TourId).ToList();
                        Tours = Tours.Where(x => IdForFilter.Contains(x.Id)).ToList();
                        break;
                    default:
                        break;
                }
            }

            if(CheckOrNot)
            {
                Tours = Tours.Where(x => x.IsActual[0]==true).ToList();
            }

            if(FindTour!=null && FindTour != "")
            {
                Tours = Tours.Where(x=>x.Name.ToUpper().Contains(FindTour.ToString().ToUpper()) || x.DescriptionWithoutNull.ToUpper().Contains(FindTour.ToString().ToUpper())).ToList();
            }

            if(SelectedSort != " ")
            {
                switch(SelectedSort)
                {
                    case "  ":
                        Tours = Tours.OrderBy(x=>x.Price).ToList();
                        break;
                    case "  ":
                        Tours = Tours.OrderByDescending(x => x.Price).ToList();
                        break;
                }
            }

            foreach(Models.Tourss t in Tours)
            {
                PriceOfAll += (t.Price * t.TicketCount);
            }
        }
    }
}